mod args;
mod rules;
mod utils;

use std::fmt::{Display, Formatter};
use eyre::Result;
use clap::Parser;
use regex::{Captures, Regex};
use walkdir::{DirEntry, WalkDir};

use args::Args;
use rules::*;
use utils::status_icon::*;

const VIDEO_EXTENSIONS: [&str; 14] = ["mkv", "mp4", "avi", "webm", "mov", "wmv", "flv", "ogg", "ogv", "yuv", "amv", "mpg", "mpeg", "m4v"];
pub(crate) const SERIES_REGEX: &str = r"^(?P<title>.*?)(?P<titleSeparator>\s-\s?)?(?P<seasonPrefix>[Ss]|\s|\.)(?P<season>\d{1,3})(?P<episodePrefix>[Ee]|[Xx]|[Ss])(?P<episode>\d{1,3})([Ee](?P<episode2>\d{2,3}))?((?P<nameSeparator>\s-\s)?(?P<name>.+))?\.(?P<ext>...)$";
// const MOVIE_REGEX: &str = r"^(?P<title>.+)\s(?P<year>\(\d{4}\))\s(?P<resolution>\[.+\])\.(?P<ext>...)$";

struct Stats {
    success: usize,
    warning: usize,
    error: usize,
    files: usize,
}

impl Display for Stats {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "Files: {} - Success: {} - Warnings: {} - Errors: {}",
            self.files,
            self.success,
            self.warning,
            self.error)
    }
}

struct Context {
    current_folder: String,
    has_printed_folder: bool,
}

fn main() -> Result<()> {
    let params = Args::parse();

    println!("{}", params.path);

    let mut stats = Stats {
        files: 0,
        success: 0,
        warning: 0,
        error: 0,
    };

    let mut ctx = Context {
        current_folder: "".to_string(),
        has_printed_folder: false,
    };

    let regex = Regex::new(SERIES_REGEX).unwrap();

    for entry in WalkDir::new(params.path) {
        if let Ok(file) = entry {
            check_file(file, &mut stats, &mut ctx, &regex, params.no_emoji, params.show_success, params.episode_name);
        }
    }

    println!("{}", stats);

    Ok(())
}

fn check_file(file: DirEntry, stats: &mut Stats, ctx: &mut Context, regex: &Regex, no_emoji: bool, show_success: bool, episode_name: bool) -> () {
    let file_type = file.file_type();
    if file_type.is_file() {
        let filename = file.file_name().to_str().unwrap();
        match lint_file_name(&file, filename, regex, episode_name) {
            ComplianceStatus::NotMatched => {
                stats.files += 1;
                stats.error += 1;
                if !ctx.has_printed_folder {
                    println!("\n\n=== {} {}", get_status_icon(StatusIcon::Directory, no_emoji), ctx.current_folder);
                    ctx.has_printed_folder = true;
                }
                println!("{} {} --- (Failed to match)", get_status_icon(StatusIcon::Failure, no_emoji), filename);
            }
            ComplianceStatus::NonCompliant(reason) => {
                stats.files += 1;
                if !ctx.has_printed_folder {
                    println!("\n\n=== {} {}", get_status_icon(StatusIcon::Directory, no_emoji), ctx.current_folder);
                    ctx.has_printed_folder = true;
                }
                stats.warning += 1;
                println!("{} {} --- ({})", get_status_icon(StatusIcon::Warning, no_emoji), filename, reason);
            }
            ComplianceStatus::Compliant => {
                stats.files += 1;
                stats.success += 1;
                if show_success {
                    println!("{} {}", get_status_icon(StatusIcon::Success, no_emoji), filename);
                }
            }
            ComplianceStatus::Ignored => {}
        }
    } else if file_type.is_dir() {
        ctx.current_folder = file.path().display().to_string();
        ctx.has_printed_folder = false;
    }
}

const SERIES_RULES_WITHOUT_NAME: [fn(&str, &Captures) -> Option<NonCompliantReason>; 3] = [
    DashInTitle::check,
    EpisodeMarker::check,
    HasFluff::check,
];

const SERIES_RULES_WITH_NAME: [fn(&str, &Captures) -> Option<NonCompliantReason>; 5] = [
    DashInTitle::check,
    EpisodeMarker::check,
    HasFluff::check,
    HasEpisodeName::check,
    MissingNameSeparator::check
];

fn lint_file_name(file: &DirEntry, filename: &str, regex: &Regex, episode_name: bool) -> ComplianceStatus {
    if let Some(ext) = file.path().extension() {
        if !VIDEO_EXTENSIONS.contains(&ext.to_str().unwrap()) {
            return ComplianceStatus::Ignored;
        }
    } else {
        return ComplianceStatus::Ignored;
    }

    let captures = regex.captures(filename);

    if captures.is_none() {
        return ComplianceStatus::NotMatched;
    }

    let captures = captures.unwrap();

    if episode_name {
        check_rules(SERIES_RULES_WITH_NAME, filename, &captures)
    } else {
        check_rules(SERIES_RULES_WITHOUT_NAME, filename, &captures)
    }
}

fn check_rules<const N: usize>(validation_rules: [fn(&str, &Captures) -> Option<NonCompliantReason>; N], filename: &str, captures: &Captures) -> ComplianceStatus {
    for rule in validation_rules {
        if let Some(reason) = rule(filename, &captures) {
            return ComplianceStatus::NonCompliant(reason);
        }
    }

    ComplianceStatus::Compliant
}